@@ -1,3 +1,4 @@
+use std::collections::HashSet;
 use std::env::{args, current_dir};
 use std::fs::read_dir;
 use std::path::{Path, PathBuf};
@@ -6,6 +7,8 @@ use std::process::{self, Child, Command};
 use anyhow::{Context, Result};
 use clap::Parser;
 
+const DEFAULT_SKIP_DIR_NAMES: [&str; 3] = [".git", ".rustup", ".cargo"];
+
 fn main() -> Result<()> {
     let mut args = args();
     if let Some("clean-recursive") = std::env::args().skip(1).next().as_deref() {
@@ -27,6 +30,9 @@ struct Args {
     /// Recursive serarch depth limit
     #[clap(long, default_value_t = 64)]
     depth: usize,
+    /// Skip directories with specified names. (if empty, '.git' and '.cargo')
+    #[clap(long)]
+    skips: Option<Vec<String>>,
     /// Target directory
     path: Option<PathBuf>,
 }
@@ -38,6 +44,16 @@ impl Args {
             release: self.release,
         };
 
+        let skips: HashSet<String> = if let Some(ref skips) = self.skips {
+            skips.iter().cloned().collect()
+        } else {
+            let mut skips = HashSet::new();
+            for n in DEFAULT_SKIP_DIR_NAMES {
+                skips.insert(n.to_string());
+            }
+            skips
+        };
+
         let depth = self.depth;
 
         let path = if let Some(path) = self.path.clone() {
@@ -48,7 +64,7 @@ impl Args {
 
         let mut children = Vec::new();
 
-        process_dir(path, depth, delete_mode, &mut children)?;
+        process_dir(path, depth, &skips, delete_mode, &mut children)?;
 
         for mut child in children {
             if let Err(e) = child.wait() {
@@ -63,6 +79,7 @@ impl Args {
 fn process_dir(
     path: PathBuf,
     depth: usize,
+    skips: &HashSet<String>,
     del_mode: DeleteMode,
     children: &mut Vec<Child>,
 ) -> Result<()> {
@@ -70,6 +87,12 @@ fn process_dir(
         return Ok(());
     }
 
+    if let Some(Some(dir_name)) = path.file_name().map(|n| n.to_str()) {
+        if skips.contains(dir_name) {
+            return Ok(());
+        }
+    }
+
     detect_and_clean(&path, del_mode, children)
         .with_context(|| format!("cleaning directory {:?}", path.display()))?;
 
@@ -78,7 +101,7 @@ fn process_dir(
     for entry in rd {
         let entry = entry?;
         if entry.file_type()?.is_dir() {
-            if let Err(e) = process_dir(entry.path(), depth - 1, del_mode, children) {
+            if let Err(e) = process_dir(entry.path(), depth - 1, skips, del_mode, children) {
                 eprintln!("{:#}", e);
             }
         }
@@ -88,12 +111,12 @@ fn process_dir(
 }
 
 fn detect_and_clean(path: &Path, del_mode: DeleteMode, children: &mut Vec<Child>) -> Result<()> {
-    let should_clean = path.join("Cargo.toml").is_file() && path.join("target").is_dir();
-    if !should_clean {
+    let is_cargo_dir = path.join("Cargo.toml").is_file();
+    if !is_cargo_dir {
         return Ok(());
     }
 
-    eprintln!("Cleaning {:?}", path);
+    eprintln!("Checking {:?}", path);
 
     if del_mode.do_all() {
         children.push(spawn_cargo_clean(path, &[])?);
